﻿#region Smart-Tech (c) 2016
//=============================================================================
// This file is part of the example provided for the Smart-RPC.net library. You 
// can distribute it under the terms of the GNU General Public License as 
// published by the Free Software Foundation, version 3 of the License.
//
// This example and the Smart-RPC.library are distributed in the hope that it 
// will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty 
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the GNU General 
// Public License for more details.
// 
// See <http://www.gnu.org/licenses/> for a copy of the GNU General Public 
// License.
// 
// This disclaimer should remain unchanged.
//=============================================================================
#endregion

using Shared;
using SmartTech.SmartComponents.Communication;
using SmartTech.SmartComponents.Operational;
using SmartTech.SmartComponents.SmartRpc;
using System;
using System.Collections.ObjectModel;
using System.Diagnostics;
using System.Net;
using System.Net.Sockets;
using System.Threading;
using System.Threading.Tasks;

namespace Client
{
    class Program
    {
        static void Main(string[] args)
        {
            SmartTech.SmartComponents.Operational.Logger.Connect(ConsoleLogger.Instance);
            //Logger.MinimumLogLevel = EnumSeverity.Verbose;

            // The custom types which are transferred via the RPC path need to be 
            // registered here. Regular system types like Int32, decimal, are implicitly 
            // already registered. This works both for the DataDictionaryMessageConverter 
            // and for the SerializableMessageConverter.
            KnownTypes.RegisterKnownType(typeof(Rectangle));
            KnownTypes.RegisterKnownType(typeof(Collection<Rectangle>));
            KnownTypes.RegisterKnownType(typeof(CallbackArgs<Int64>));

            IPAddress localIpAddress = IPAddressHelper.GetLocalIPAddress();

            // ClientManager for connection with support for JsonMessages.
            Logger.Log(
                EnumSeverity.Info,
                "=== JsonMessageConverter ===");

            ClientManager clientManager1 = CreateConnectedClientManager(
                new TcpEndPoint(localIpAddress.ToString(), 12000),
                new JsonMessageConverter());

            ExampleServiceAdapter serviceAdapter1 = new ExampleServiceAdapter(clientManager1);

            clientManager1.RegisterCallbackReceiver<ICallbackContract>(
                new CallbackReceiver("CallbackReceiver #1"));

            ExecuteNotificationExample(serviceAdapter1);
            ExecuteSerializableExample(serviceAdapter1);

            MessageConverter messageConverter = null; 

            // ClientManager for connection with support for the DataDictionaryMessages.
            // This allows for easy serialization for complex data types.
            Logger.Log(
                EnumSeverity.Info,
                "=== SerializableMessageConverter ===");

            messageConverter = new SerializableMessageConverter();

            // This client manager will listen to broadcasts originating from the 
            // service. When the corresponding endpoint is detected (based on the 
            // type of CommunicationBinding and type of MessageConverter) the 
            // connection is automatically made without direct knowledge where 
            // the services run. Note that the port 9999 is used to relate the 
            // client with the remote service.
            IPEndPoint endPoint = new IPEndPoint(IPAddress.Any, 9999);
            UdpBroadcastReceiver broadcastReceiver =
                new UdpBroadcastReceiver(
                    messageConverter, 
                    endPoint);

            // This client manager used the passed broadcast receiver to detect 
            // the corresponding endpoint which is broadcasted by the remote 
            // service.
            DiscoveryClientManager clientManager2 = new DiscoveryClientManager(
                new TcpCommunicationBinding(EnumBindingMode.Client),
                messageConverter,
                broadcastReceiver);

            Logger.Log(
                EnumSeverity.Info,
                "Waiting for a broadcast from the remote side");

            // Trigger the client manager to start the connection. This will 
            // trigger the listening for the broadcasts from the remote service.
            clientManager2.Connect();

            ExampleServiceAdapter serviceAdapter2 = new ExampleServiceAdapter(clientManager2);

            clientManager2.RegisterCallbackReceiver<ICallbackContract>(
                new CallbackReceiver("CallbackReceiver #2"));

            // We need to wait synchronously for the connection with the discovered 
            // remote service endpoint.
            ManualResetEvent remoteSideConnected = new ManualResetEvent(false);

            serviceAdapter2.ConnectedStateChanged += (sender, e) =>
            {
                if (e.Parameter)
                {
                    remoteSideConnected.Set();
                }
            };

            if (remoteSideConnected.WaitOne(5000))
            {
                Logger.Log(
                    EnumSeverity.Info,
                    "We received a broadcast from the remote side and are now connected");

                ExecuteNotificationExample(serviceAdapter2);
                ExecuteSerializableExample(serviceAdapter2);

            }
            else
            {
                Logger.Log(
                    EnumSeverity.Error,
                    "No broadcast was received, please check the IP address used " +
                    "at the service side for the UdpBroadcastSender");
            }

            // ClientManager for connection with support for the SerializableMessages.
            // When custom types implement ISerializable they can be communicated 
            // between client and service(s). Look at the Rectangle for a basic 
            // implementation.
            Logger.Log(
                EnumSeverity.Info,
                "=== DataDictionaryMessageConverter ===");

            // This message converter is updated with 2 data processors to show 
            // how to extend the processing of the messages.
            messageConverter = new DataDictionaryMessageConverter();
            messageConverter.RegisterDataProcessor(new EncryptionDataProcessor("TEST_PASSWORD"));
            messageConverter.RegisterDataProcessor(new CompressionDataProcessor());

            ClientManager clientManager3 = CreateConnectedClientManager(
                new TcpEndPoint(localIpAddress.ToString(), 12002),
                messageConverter);

            ExampleServiceAdapter serviceAdapter3 = new ExampleServiceAdapter(clientManager3);

            clientManager3.RegisterCallbackReceiver<ICallbackContract>(
                new CallbackReceiver("CallbackReceiver #3"));

            ExecuteNotificationExample(serviceAdapter3);
            ExecuteSerializableExample(serviceAdapter3);

            // Try to request a large message multiple times.
            for (int i = 0; i < 5; i++)
            {
                Stopwatch stopwatch = Stopwatch.StartNew();
                var responseCreateRectangleCollection = serviceAdapter3.CreateRectangleCollection(500000);

                stopwatch.Stop();

                if (responseCreateRectangleCollection.Result == EnumServiceResult.Success)
                {
                    Logger.Log(
                        EnumSeverity.Info, 
                        "Creating 500000 rectangles and retrieving them took: {0} milliseconds", 
                        stopwatch.ElapsedMilliseconds);
                }
                else
                {
                    Logger.Log(
                        EnumSeverity.Info,
                        "Creating 500000 rectangles and retrieving them failed, result: {0}",
                        responseCreateRectangleCollection.Result);
                }
            }

            Logger.Log(
                EnumSeverity.Info,
                "=== Finished ===");

            Console.ReadLine();
        }

        /// <summary>
        /// Create a client manager which has an active connection with the remote 
        /// side.
        /// </summary>
        /// <param name="tcpEndPoint">The endpoint with which we should connect.
        /// </param>
        /// <param name="messageConverter">The message converter used for this 
        /// specific connection.</param>
        /// <returns>The ClientManager.</returns>
        private static ClientManager CreateConnectedClientManager(
            TcpEndPoint tcpEndPoint,
            MessageConverter messageConverter)
        {
            ManualResetEvent connected = new ManualResetEvent(false);

            // The connection timeout on the communication binding is set to 1000 
            // milliseconds. This ensures that the connection is monitored via keep 
            // alive messages which are sent every 1000 milliseconds. One message 
            // is allowed to be missed and effectively a connection is severed 
            // when 2 messages are missed which totals to 2000 milliseconds.
            ClientManager clientManager =
                new ClientManager(
                    new TcpCommunicationBinding(EnumBindingMode.Client, 10000),
                    messageConverter);

            // We need to wait for the client manager to have an active connection 
            // with the remote side.
            clientManager.ConnectedStateChanged += (sender, e) =>
            {
                if (e.Parameter)
                {
                    connected.Set();
                }
            };

            clientManager.Connect(tcpEndPoint);

            connected.WaitOne(5000);

            return clientManager;
        }

        /// <summary>
        /// This contains an example that will have this client subscribe onto 
        /// notifications and then trigger multiple adds which result in both 
        /// a response and a notification.
        /// </summary>
        /// <param name="serviceAdapter">The service adapter providing the 
        /// communication channel with the service.</param>
        public static void ExecuteNotificationExample(ExampleServiceAdapter serviceAdapter)
        {
            serviceAdapter.Subscribe();

            for (int i = 1; i <= 5; i++)
            {
                var responseAdd = serviceAdapter.Add(i, i);

                // Notice that an operation contains both a result indicating 
                // the success of the operation and a parameter containing an 
                // actual result.
                if (responseAdd.Result == EnumServiceResult.Success)
                {
                    Logger.Log(
                        EnumSeverity.Info,
                        "Adding {0} and {0} resulted in {1}",
                        i,
                        responseAdd.Parameter);
                }
                else
                {
                    Logger.Log(
                        EnumSeverity.Warning,
                        "Adding {0} and {0} failed, {1}",
                        i,
                        responseAdd.Result);
                }
            }

            serviceAdapter.Unsubscribe();
        }

        /// <summary>
        /// This contains 2 different operations which are called on the service 
        /// and which displays that system types (currently byte, byte[], String, 
        /// Boolean, DateTime, TimeSpan, Single, Double, UInt16, UInt32, UInt64, 
        /// Int16, Int32, Int64) can be serialized. It also shows how to have 
        /// custom types implement ISerializable after which they can also cross 
        /// the boundary between client and service.
        /// </summary>
        /// <param name="serviceAdapter">The service adapter providing the 
        /// communication channel with the service.</param>
        public static void ExecuteSerializableExample(ExampleServiceAdapter serviceAdapter)
        {
            // Show that system types can be passed onto the service and back. For 
            // this example the Single type is used.
            var responseSubtract = serviceAdapter.Subtract(3f, 2.5f);

            if (responseSubtract.Result == EnumServiceResult.Success)
            {
                Logger.Log(
                    EnumSeverity.Info,
                    "Subtract(3, 2.5) returned: {0}",
                    responseSubtract.Parameter);
            }
            else
            {
                Logger.Log(
                    EnumSeverity.Warning,
                    "Subtract failed: {0}",
                    responseSubtract.Result);
            }

            // Show that a custom type can be created at the service side and 
            // be received at the client side. The Parameter property contains 
            // an instance of the custom Rectangle type.
            var responseCreateRectangle = serviceAdapter.CreateRectangle(5.25f, 9.5f);

            if (responseCreateRectangle.Result == EnumServiceResult.Success)
            {
                Logger.Log(
                    EnumSeverity.Info,
                    "CreateRectangle returned: {0}",
                    responseCreateRectangle.Parameter);
            }
            else
            {
                Logger.Log(
                    EnumSeverity.Warning,
                    "CreateRectangle failed: {0}",
                    responseCreateRectangle.Result);
            }
        }
    }
}
