﻿#region Smart-Tech (c) 2016
//=============================================================================
// This file is part of the example provided for the Smart-RPC.net library. You 
// can distribute it under the terms of the GNU General Public License as 
// published by the Free Software Foundation, version 3 of the License.
//
// This example and the Smart-RPC.library are distributed in the hope that it 
// will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty 
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the GNU General 
// Public License for more details.
// 
// See <http://www.gnu.org/licenses/> for a copy of the GNU General Public 
// License.
// 
// This disclaimer should remain unchanged.
//=============================================================================
#endregion

using Shared;
using SmartTech.SmartComponents.Communication;
using SmartTech.SmartComponents.Operational;
using SmartTech.SmartComponents.SmartRpc;
using System;
using System.Collections.ObjectModel;
using System.Net;

namespace Example
{
    /// <summary>
    /// This service is provided to show how a service can be created and how 
    /// methods can be added. It also shows how clients can be subscribed for 
    /// notifications / callbacks.
    /// </summary>
    internal class ExampleService : RpcService
    {
        // All class constructors, initialize, terminate and dispose methods.
        #region Construct / Init / Terminate / Dispose

        /// <summary>
        /// Default constructor for class
        /// </summary>
        public ExampleService()
        {
            _callbackRegistrations = new CallbackRegistrations("ICallbackContract");

            // Register the types required for this service in relation to the 
            // (de)serialization.
            KnownTypes.RegisterKnownType(typeof(CallbackArgs<Int64>));
            KnownTypes.RegisterKnownType(typeof(Response<Int64>));
        }

        /// <summary>
        /// Dispose the managed resources.
        /// </summary>
        protected override void DisposeManagedResources()
        {
            base.DisposeManagedResources();

            _callbackRegistrations.Dispose();
        }

        #endregion

        // Use this region to implement interface members for the given interface.
        #region <interface name> Members
        #endregion

        // Properties for this class.
        #region Properties
        #endregion

        // Events for this class.
        #region Events
        #endregion

        // All public methods for this class.
        #region Methods

        /// <summary>
        /// Register this instance as an RPC service with the passed service host.
        /// </summary>
        /// <param name="rpcServiceHost">The service host to register with.</param>
        /// <returns>The result of the registration.</returns>
        public override EnumOperationResult RegisterAsService(RpcServiceHost rpcServiceHost)
        {
            EnumOperationResult result = EnumOperationResult.PreconditionsFailed;

            if (rpcServiceHost != null)
            {
                // We need to provide the local IP address since it is not possible 
                // to retrieve in the framework due to the reduced framework and the 
                // missing of required calls.
                IPAddress localIpAddress = IPAddressHelper.GetLocalIPAddress();

                if (localIpAddress != null)
                {
                    // First we need to register this service instance so that it 
                    // can be used.
                    rpcServiceHost.RegisterService(this);

                    // The custom types which are transferred via the RPC path need to be 
                    // registered here. Regular system types like Int32, decimal, are implicitly 
                    // already registered. This works both for the DataDictionaryMessageConverter 
                    // and for the SerializableMessageConverter.
                    KnownTypes.RegisterKnownType(typeof(Rectangle));
                    KnownTypes.RegisterKnownType(typeof(Collection<Rectangle>));

                    MessageConverter messageConverter = null;

                    // Endpoint 1, using the JsonMessageConverter.
                    // The connection timeout on the communication binding is set to 1000 
                    // milliseconds. This ensures that the connection is monitored via keep 
                    // alive messages which are sent every 1000 milliseconds. One message 
                    // is allowed to be missed and effectively a connection is severed 
                    // when 2 messages are missed which totals to 2000 milliseconds.
                    messageConverter = new JsonMessageConverter();

                    rpcServiceHost.RegisterEndpoint(
                        new TcpEndPoint(localIpAddress.ToString(), 12000),
                        new TcpCommunicationBinding(EnumBindingMode.Service, 1000),
                        messageConverter);

                    // Endpoint 2, using the SerializableMessageConverter.
                    messageConverter = new SerializableMessageConverter();

                    rpcServiceHost.RegisterEndpoint(
                        new TcpEndPoint(localIpAddress.ToString(), 12001),
                        new TcpCommunicationBinding(EnumBindingMode.Service, 6000),
                        messageConverter);

                    // Endpoint 3, using the DataDictionaryMessageConverter.
                    // This message converter is updated with 2 data processors to show 
                    // how to extend the processing of the messages.
                    messageConverter = new DataDictionaryMessageConverter();
                    messageConverter.RegisterDataProcessor(new EncryptionDataProcessor("TEST_PASSWORD"));
                    messageConverter.RegisterDataProcessor(new CompressionDataProcessor());

                    rpcServiceHost.RegisterEndpoint(
                        new TcpEndPoint(localIpAddress.ToString(), 12002),
                        new TcpCommunicationBinding(EnumBindingMode.Service, 10000),
                        messageConverter);
                }
                else
                {
                    Logger.Log(
                        EnumSeverity.Warning,
                        "Unable to retrieve the local IP address");

                    result = EnumOperationResult.InternalFailure;
                }
            }

            return result;
        }

        /// <summary>
        /// Add the passed integers and return them.
        /// </summary>
        /// <param name="a">First integer.</param>
        /// <param name="b">Second integer.</param>
        /// <returns>The result, this needs to be a class due to internal 
        /// requirements.</returns>
        public Int64 Add(Int64 a, Int64 b)
        {
            Int64 result = a + b;

            _callbackRegistrations.Fire(
                "OnIntegersAdded", 
                new CallbackArgs<Int64>(result));

            return result;
        }

        public Double Subtract(Double a, Double b)
        {
            return a - b;
        }

        public Rectangle CreateRectangle(Double height, Double width)
        {
            return new Rectangle(height, width);
        }

        public Collection<Rectangle> CreateRectangleCollection(Int32 numberOfRectangles)
        {
            Collection<Rectangle> rectangleCollection = new Collection<Rectangle>();

            if (numberOfRectangles > 0)
            {
                while (numberOfRectangles > 0)
                {
                    rectangleCollection.Add(
                        new Rectangle(numberOfRectangles, numberOfRectangles * 2));

                    numberOfRectangles--;
                }
            }

            return rectangleCollection;
        }

        /// <summary>
        /// Subscribe the caller onto the callbacks originating from this service.
        /// </summary>
        public void Subscribe()
        {
            _callbackRegistrations.Add();
        }

        /// <summary>
        /// Unsubscribe the caller from the callbacks originating from this service.
        /// </summary>
        public void Unsubscribe()
        {
            _callbackRegistrations.Remove();
        }

        #endregion

        // All protected / private methods for this class.
        #region Helper methods
        #endregion

        // All event handlers for this class.
        #region Event handlers
        #endregion

        // Private fields
        #region Fields

        /// <summary>
        /// Used to register the subscribers for the callbacks we can fire.
        /// </summary>
        private CallbackRegistrations _callbackRegistrations = null;

        #endregion
    }
}
