﻿#region Smart-Tech (c) 2016
//=============================================================================
// This file is part of the example provided for the Smart-RPC.net library. You 
// can distribute it under the terms of the GNU General Public License as 
// published by the Free Software Foundation, version 3 of the License.
//
// This example and the Smart-RPC.library are distributed in the hope that it 
// will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty 
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the GNU General 
// Public License for more details.
// 
// See <http://www.gnu.org/licenses/> for a copy of the GNU General Public 
// License.
// 
// This disclaimer should remain unchanged.
//=============================================================================
#endregion

using SmartTech.SmartComponents.Operational;
using System;

namespace Shared
{
    /// <summary>
    /// Provide an implementation which is used by the logging mechanism used 
    /// by the Smart-RPC.net library. Messages are logged to the console.
    /// </summary>
    public class ConsoleLogger : ILogger
    {
        // All class constructors, initialize, terminate and dispose methods.
        #region Construct / Init / Terminate / Dispose

        /// <summary>
        /// Default constructor for class
        /// </summary>
        private ConsoleLogger()
        {
        }

        #endregion

        // Use this region to implement interface members for the given interface.
        #region ILogger Members

        /// <summary>
        /// Log a message for the specified module.
        /// </summary>
        /// <param name="severity">The severity of the message.</param>
        /// <param name="methodName">The name of the method from which the message 
        /// originated.</param>
        /// <param name="message">The message itself.</param>
        /// <param name="args">Optional extra parameters used in the message.
        /// </param>
        public void Log(EnumSeverity severity, string methodName, string message, params object[] args)
        {
            switch (severity)
            {
                case EnumSeverity.Info:
                    Console.ForegroundColor = ConsoleColor.Green;
                    break;
                case EnumSeverity.Warning:
                    Console.ForegroundColor = ConsoleColor.Yellow;
                    break;
                case EnumSeverity.Error:
                    Console.ForegroundColor = ConsoleColor.Red;
                    break;
                case EnumSeverity.Fatal:
                    Console.ForegroundColor = ConsoleColor.Magenta;
                    break;
            }

            Console.WriteLine(string.Format("{0} {1}: {2}",
                DateTime.Now.ToString("HH:mm:ss.fff"),
                severity,
                string.Format(message, args)));

            Console.ResetColor();
        }

        /// <summary>
        /// Log the message.
        /// </summary>
        /// <param name="loggingMessage">The message containing the logging 
        /// information.</param>
        public void Log(LoggingMessage loggingMessage)
        {
            Log(
                loggingMessage.Severity, 
                loggingMessage.MethodName, 
                loggingMessage.Message);
        }

        #endregion

        // Properties for this class.
        #region Properties

        /// <summary>
        /// Return the singleton instance.
        /// </summary>
        public static ConsoleLogger Instance
        {
            get
            {
                if (_instance == null)
                {
                    lock (_lockObject)
                    {
                        if (_instance == null)
                        {
                            _instance = new ConsoleLogger();
                            Logger.Connect(_instance);
                        }
                    }
                }

                return _instance;
            }
        }

        #endregion

        // Events for this class.
        #region Events
        #endregion

        // All public methods for this class.
        #region Methods
        #endregion

        // All protected / private methods for this class.
        #region Helper methods
        #endregion

        // All event handlers for this class.
        #region Event handlers
        #endregion

        // Private fields
        #region Fields

        /// <summary>
        /// Required for threadsafe access to the internals.
        /// </summary>
        private static readonly object _lockObject = new object();

        /// <summary>
        /// The singleton instance.
        /// </summary>
        private static ConsoleLogger _instance = null;

        #endregion
    }
}
