﻿#region Smart-Tech (c) 2016
//=============================================================================
// This file is part of the example provided for the Smart-RPC.net library. You 
// can distribute it under the terms of the GNU General Public License as 
// published by the Free Software Foundation, version 3 of the License.
//
// This example and the Smart-RPC.library are distributed in the hope that it 
// will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty 
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the GNU General 
// Public License for more details.
// 
// See <http://www.gnu.org/licenses/> for a copy of the GNU General Public 
// License.
// 
// This disclaimer should remain unchanged.
//=============================================================================
#endregion

using SmartTech.SmartComponents.Operational;
using SmartTech.SmartComponents.SmartRpc;
using System;
using System.IO;
using System.IO.Compression;

namespace Shared
{
    /// <summary>
    /// This is used to compress / decompress the data contained in the messages 
    /// used between client and remote side. The NuGet package Microsoft.BCL.Compression 
    /// is used. However with the small packages used for the communication the 
    /// compressed data is sometimes larger than the uncompressed data.
    /// </summary>
    public class CompressionDataProcessor : DataProcessor
    {
        // All class constructors, initialize, terminate and dispose methods.
        #region Construct / Init / Terminate / Dispose

        /// <summary>
        /// Default constructor for class
        /// </summary>
        public CompressionDataProcessor()
        {
        }

        #endregion

        // Use this region to implement interface members for the given interface.
        #region <interface name> Members
        #endregion

        // Properties for this class.
        #region Properties
        #endregion

        // Events for this class.
        #region Events
        #endregion

        // All public methods for this class.
        #region Methods

        /// <summary>
        /// Have the passed data compressed.
        /// </summary>
        /// <param name="data">The data to compress.</param>
        /// <returns>The result, when <b>true</b> the parameter contains the 
        /// compressed data.</returns>
        public override ParameterResponse<EnumOperationResult, byte[]> Process(byte[] data)
        {
            ParameterResponse<EnumOperationResult, byte[]> response =
                new ParameterResponse<EnumOperationResult, byte[]>(
                    EnumOperationResult.PreconditionsFailed, null);

            if (data != null)
            {
                if (!SafeExecute(() =>
                {
                    using (MemoryStream sourceStream = new MemoryStream(data))
                    {
                        using (MemoryStream targetStream = new MemoryStream())
                        {
                            using (GZipStream compressionStream = new GZipStream(targetStream, CompressionMode.Compress, true))
                            {
                                // Encrypt the source data.
                                sourceStream.CopyTo(compressionStream);
                            }

                            response.Parameter = targetStream.ToArray();

                            Single compressionRate = response.Parameter.Length / (Single)data.Length;

                            if (compressionRate < 1)
                            {
                                Logger.Log(
                                    EnumSeverity.Verbose,
                                    "Compressed data, original size [{0}] bytes / compressed size [{1}] bytes",
                                    data.Length,
                                    response.Parameter.Length);
                            }
                            else
                            {
                                Logger.Log(
                                    EnumSeverity.Warning,
                                    "Compressed data Ratio [{0:0.00}], original size {1} bytes / compressed size {2} bytes",
                                    compressionRate,
                                    data.Length,
                                    response.Parameter.Length);
                            }

                            response.Result = EnumOperationResult.Success;
                        }
                    }
                }, false))
                {
                    response.Result = EnumOperationResult.InternalFailure;
                }
            }

            return response;
        }

        /// <summary>
        /// Have the passed data uncompressed.
        /// </summary>
        /// <param name="data">The data to uncompress.</param>
        /// <returns>The result, when <b>true</b> the parameter contains the 
        /// uncompressed data.</returns>
        public override ParameterResponse<EnumOperationResult, byte[]> RevertProcess(byte[] data)
        {
            ParameterResponse<EnumOperationResult, byte[]> response =
                new ParameterResponse<EnumOperationResult, byte[]>(
                    EnumOperationResult.PreconditionsFailed, null);

            if (data != null)
            {
                if (!SafeExecute(() =>
                {
                    using (MemoryStream sourceStream = new MemoryStream(data))
                    {
                        using (MemoryStream targetStream = new MemoryStream())
                        {
                            using (GZipStream compressionStream = new GZipStream(sourceStream, CompressionMode.Decompress))
                            {
                                // Decrypt the source data.
                                compressionStream.CopyTo(targetStream);
                            }

                            response.Parameter = targetStream.ToArray();

                            Logger.Log(
                                EnumSeverity.Verbose,
                                "Uncompressed data, compressed size [{0}] bytes / original size [{1}] bytes",
                                data.Length,
                                response.Parameter.Length);

                            response.Result = EnumOperationResult.Success;
                        }
                    }
                }, false))
                {
                    response.Result = EnumOperationResult.InternalFailure;
                }
            }

            return response;
        }

        #endregion

        // All protected / private methods for this class.
        #region Helper methods
        #endregion

        // All event handlers for this class.
        #region Event handlers
        #endregion

        // Private fields
        #region Fields
        #endregion
    }
}
