﻿#region Smart-Tech (c) 2016
//=============================================================================
// This file is part of the example provided for the Smart-RPC.net library. You 
// can distribute it under the terms of the GNU General Public License as 
// published by the Free Software Foundation, version 3 of the License.
//
// This example and the Smart-RPC.library are distributed in the hope that it 
// will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty 
// of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the GNU General 
// Public License for more details.
// 
// See <http://www.gnu.org/licenses/> for a copy of the GNU General Public 
// License.
// 
// This disclaimer should remain unchanged.
//=============================================================================
#endregion

using Newtonsoft.Json;
using Newtonsoft.Json.Linq;
using SmartTech.SmartComponents.Communication;
using SmartTech.SmartComponents.Operational;
using SmartTech.SmartComponents.SmartRpc;
using System;
using System.Diagnostics.Contracts;
using System.Text;

namespace Shared
{
    /// <summary>
    /// This message converter is provided as an example to show how to convert 
    /// the internal used Rpc messages to and from the Json messages which are 
    /// communicated between service and client.
    /// </summary>
    public class JsonMessageConverter : MessageConverter
    {
        // All class constructors, initialize, terminate and dispose methods.
        #region Construct / Init / Terminate / Dispose

        /// <summary>
        /// Default constructor for class
        /// </summary>
        public JsonMessageConverter()
        {
        }

        #endregion

        // Use this region to implement interface members for the given interface.
        #region <interface name> Members
        #endregion

        // Properties for this class.
        #region Properties
        #endregion

        // Events for this class.
        #region Events
        #endregion

        // All public methods for this class.
        #region Methods
        #endregion

        // All protected / private methods for this class.
        #region Helper methods

        /// <summary>
        /// Convert the passed rpc response into the generic message so that it 
        /// can be sent to the remote client.
        /// </summary>
        /// <param name="rpcResponse">The response to convert.</param>
        /// <returns>The result, if <b>true</b> the parameter contains the 
        /// message.</returns>
        protected override ParameterResponse<bool, Message> InternalConvertIntoMessage(
            RpcResponse rpcResponse)
        {
            Contract.Requires(rpcResponse != null);

            ParameterResponse<bool, Message> response =
                new ParameterResponse<bool, Message>(false, null);

            if (rpcResponse != null)
            {
                SafeExecute(() =>
                {
                    response.Parameter = new ResponseMessage(
                        Encoding.UTF8.GetBytes(
                            JsonConvert.SerializeObject(rpcResponse)));

                    response.Result = true;
                }, false);
            }

            return response;
        }

        /// <summary>
        /// Convert the passed rpc request into the generic message so that it 
        /// can be sent to the remote client.
        /// </summary>
        /// <param name="rpcRequest">The request to convert.</param>
        /// <returns>The result, if <b>true</b> the parameter contains the 
        /// message.</returns>
        protected override ParameterResponse<bool, Message> InternalConvertIntoMessage(
            RpcRequest rpcRequest)
        {
            Contract.Requires(rpcRequest != null);

            ParameterResponse<bool, Message> response =
                new ParameterResponse<bool, Message>(false, null);

            if (rpcRequest != null)
            {
                SafeExecute(() =>
                {
                    response.Parameter = new RequestMessage(
                        Encoding.UTF8.GetBytes(
                            JsonConvert.SerializeObject(rpcRequest)));

                    response.Result = true;
                }, false);
            }

            return response;
        }

        /// <summary>
        /// Convert the passed rpc notification into the generic message so that it 
        /// can be sent to the remote client.
        /// </summary>
        /// <param name="rpcNotification">The notification to convert.</param>
        /// <returns>The result, if <b>true</b> the parameter contains the 
        /// message.</returns>
        protected override ParameterResponse<bool, Message> InternalConvertIntoMessage(
            RpcNotification rpcNotification)
        {
            Contract.Requires(rpcNotification != null);

            ParameterResponse<bool, Message> response =
                new ParameterResponse<bool, Message>(false, null);

            if (rpcNotification != null)
            {
                SafeExecute(() =>
                {
                    response.Parameter = new NotificationMessage(
                        Encoding.UTF8.GetBytes(
                            JsonConvert.SerializeObject(rpcNotification)));

                    response.Result = true;
                }, false);
            }

            return response;
        }

        /// <summary>
        /// Convert the passed message into an RPC notification.
        /// </summary>
        /// <param name="message">The message.</param>
        /// <returns>The result.</returns>
        protected override ParameterResponse<bool, RpcNotification> InternalConvertIntoRpcNotification(
            Message message)
        {
            Contract.Requires(message != null);

            ParameterResponse<bool, RpcNotification> response =
                new ParameterResponse<bool, RpcNotification>(false, null);

            if (message != null)
            {
                response.Parameter = JsonConvert.DeserializeObject<RpcNotification>(
                    Encoding.UTF8.GetString(message.Data));

                // Rough way to replace the second level object into the correct 
                // type. Needs to be extended when dealing with other data.
                foreach (Parameter parameter in response.Parameter.Parameters)
                {
                    JObject jObject = parameter.Value as JObject;

                    if (jObject != null)
                    {
                        if (jObject["Parameter"].Type == JTokenType.Integer)
                        {
                            parameter.Value = new CallbackArgs<Int64>(jObject["Parameter"].Value<Int64>());
                        }
                    }
                }

                response.Result = true;
            }

            return response;
        }

        /// <summary>
        /// Convert the passed message into an RPC request.
        /// </summary>
        /// <param name="message">The message.</param>
        /// <returns>The result.</returns>
        protected override ParameterResponse<bool, RpcRequest> InternalConvertIntoRpcRequest(
            Message message)
        {
            Contract.Requires(message != null);

            ParameterResponse<bool, RpcRequest> response =
                new ParameterResponse<bool, RpcRequest>(false, null);

            if (message != null)
            {
                response.Parameter = JsonConvert.DeserializeObject<RpcRequest>(
                    Encoding.UTF8.GetString(message.Data));

                response.Result = true;
            }

            return response;
        }

        /// <summary>
        /// Convert the passed message into an RPC response.
        /// </summary>
        /// <param name="message">The message.</param>
        /// <returns>The result.</returns>
        protected override ParameterResponse<bool, RpcResponse> InternalConvertIntoRpcResponse(
            Message message)
        {
            Contract.Requires(message != null);

            ParameterResponse<bool, RpcResponse> response =
                new ParameterResponse<bool, RpcResponse>(false, null);

            if (message != null)
            {
                response.Parameter = JsonConvert.DeserializeObject<RpcResponse>(
                    Encoding.UTF8.GetString(message.Data));

                // Rough way to replace the second level object into the correct 
                // type. Needs to be extended when dealing with other data.
                JObject jObject = response.Parameter.Value as JObject;

                if (jObject != null)
                {
                    JToken jToken = jObject["result"];

                    if (jToken != null)
                    {
                        if (jToken.Type == JTokenType.Integer)
                        {
                            response.Parameter.Value = jToken.Value<Int64>();
                        }
                    }
                    else
                    {
                        jToken = jObject["Height"];

                        if (jToken != null)
                        {
                            // We are dealing with a Rectangle.
                            response.Parameter.Value = new Rectangle(jToken.Value<Double>(), jObject["Width"].Value<Double>());
                        }
                    }
                }

                response.Result = true;
            }

            return response;
        }

        #endregion

        // All event handlers for this class.
        #region Event handlers
        #endregion

        // Private fields
        #region Fields
        #endregion
    }
}
